% Matlab Command Examples by Berkan Solmaz, Computer Vision Lab, UCF.

close all;clear all;clc;

help ops; % shows operators and special characters
clc; % clears command window
help sin

% variables
% Variables are defined as the assignment operator.
a=17         
b='cap5415'
c=[3 4 2];      
D=5*c  - 2;             
e=c.^2 * sin(a);

% Vector (1-D matrix, array)
x = [1 2 4 5]
x(1)
x([1 4])

% Indexing into vectors
x = 1:10
x = 1:2:10
x = 10:-2:1
y = 10:-2:1
x = 1:2:10

% Vector Addition
x=[2 5]
y=[4 3]
v = x+y

% Scalar  Product
a=3
w = a*v

% Operations on Vectors
sum(x)
max(x)
min(x)
mean(x)
x.^2
x .* y
x .^ y

% Inner (dot) product
dot(x,y)
x=[1 1]
y=[-1 1]
dot(x,y)

% Matrices 1
clc
A = zeros(10, 10)
A = rand(10, 10)
eye(3)

A=rand(5,5)
A(1, 4)

A=[1 2 3; 4 5 6]
B=2*ones(3,4)
%C=A+B;

% Matrices 2
A=[1 2 3; 4 5 6]
B=2*ones(2,3)
C=A*B'

A=[7 3 6; 5 1 4;2 9 8];
eye(3)*A*eye(3)

% Matrices 3
C=A'

(A+C)'

A'+C'

% Matrices 4
A=[1 2; 3 4]
det(A)

% Matrices 5
inv(A)

A*inv(A)


% Indexing 2-D Matrices
A=102:2:150
A=reshape(A,[5 5])
whos
whos A
A(4)
A(4)
A(4,1)
help sub2ind
sub2ind([5 5],2,3)

clear;
whos
clc
close all;
memory
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Reading Images
clear all;close all;clc;

% - imread. Image reading from disk.
I1 = imread('groceries.jpg');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - imshow, imagesc. Showing images and matrices in matlab.
figure;
imshow(I1);

figure;imshow(I1(:,:,1));
figure;imshow(I1(:,:,2));
figure;imshow(I1(:,:,3));

% - rgb2gray. Convert color image to grayscale.
I2 = rgb2gray(I1);

figure;
imshow(I2);

figure;
imagesc(I2)

imwrite(I2,'groceries_gray.jpg');

[rows cols chan]=size(I2);

whos

close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% thresholding, selecting indices, initializing an array
thr=128;
I3 = zeros(rows,cols,'uint8');
I3(find(I2>thr))=70;
I3(find(I2<=thr))=0;
figure;
imshow(I3);

% - mat2gray. Convert arbitrary matrix to scaled, intensity image
I4=mat2gray(I3);
figure;
imshow(I4);

unique(I4)

% - hist. Make histogram from a matrix / image.
figure;
imhist(I2);

% - im2bw. Threshold gray image
I5=im2bw(I2,0.2); % thr 0 to 1
figure;
imshow(I5);

% - Box kernel for image smoothing
h=ones(7)/49;
I6=uint8(conv2(I2,h,'same'));
figure;imshow(I6);

% - Separable kernels
% - Computing derivatives
close all;
keyboard;
M=magic(7)
f1=[1;-1] % vertical gradient
f2=[-1 1] % horizontal gradient
a=conv2(double(M),f1,'same')
b=conv2(double(a),f2,'same')
f3=f1*f2
c=conv2(double(M),f3,'same')
sum(sum(abs(b-c)))

% - Forward, backward and central difference (I showed Alper's 5415 slide for this).
M=magic(5)
hback=[-1 1];
out=conv2(double(M),hback,'same')
hforw=[1 -1];
out=conv2(double(M),hforw,'same')
hcent=[1 0 -1];
out=conv2(double(M),hcent,'same')

% - Display derivative.
horgradI2=abs(conv2(double(I2),hcent,'same'));
figure;imshow(uint8(horgradI2));
vergradI2=abs(conv2(double(I2),hcent','same'));
figure;imshow(uint8(vergradI2));

% - Compute gradient magnitude and direction.
gradMag=sqrt(vergradI2.^2+horgradI2.^2);
figure;imshow(uint8(gradMag));
gradDir=atan2(vergradI2,horgradI2);

% - fspecial. Create arbitrary sized Gaussian kernel with different sizes and variances and their effects on smoothing (over/under smooth)
hg = fspecial('gaussian', [7 7], 1)

% - Use Gaussian kernel for smoothing
I3=conv2(double(I2),hg,'same');
imshow(uint8(I3));

% - Gaussian derivative. Wrote a one line code/formula for creating a Gaussian derivative kernel, for gradient computation.
% - I emphasized Gaussian derivative and that, in general, this should be used for gradient computation of images.
gaussder = conv2(fspecial('gaussian',[41 41],4),[1 0 -1],'valid');
figure;surf(gaussder);

% - Using matlab efficiently (i.e., matrix manipulation instead of loops).
% - Example of computing distances between all pairs of points in two sets using matrices.
a=uint8(100*rand(1,5))
b=uint8(100*rand(1,5))
diff=zeros(5,5)
for i=1:5
    diff(i,:)=double(b)-double(a(i));
end

%computing distances
diff2=repmat(double(b),5,1)-repmat(double(a'),1,5)

% - repmat. repeat matrices
a=[1 2 3 4]
repmat(a,[3 2])

% - rand. generate random numbers.
rand()
rand(5,4)

% - meshgrid. useful in creating system of equations in Anandan's etc.
[x y]=meshgrid(1:5,1:3)

% - Read and display frames of videos that were already sextracted using a utility (Virtual Dub)
inputPath = 'Sequence';
figure;
for i = 133:183
    sprintf('%04d.jpeg', i)
    imshow(imread([inputPath '\' sprintf('%04d.jpeg', i)]));
    pause(.1);
end
 
% - Read and display frames directly from AVI video.
aviinfo('street.avi')
frame25 = aviread('street.avi', 25);
whos
frame25
imshow(frame25.cdata);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% plotting points on 1D/2D
plot(0:10:101)
Q = [0, 0, 1, 1; 0, 1, 0, 2];
Q
figure
plot(Q(1,:), Q(2,:), 'o', 'LineWidth', 2)
plot(rand(100,1), rand(100,1), 'b.');
plot(rand(100,1), rand(100,1), 'r.');
figure;
close all;
plot(rand(100,1), rand(100,1), 'b.');
plot(rand(100,1), rand(100,1), 'b.');
hold on
plot(rand(100,1), rand(100,1), 'r.');
hold off

% convolution
box = ones(1,25)
smoothIx = conv2(double(I2), box);
close all;
figure; imshow(I2);
[ min(smoothIx(:)) max(smoothIx(:)) ]
figure; imshow(smoothIx, []);
box
box'
smoothI2 = conv2(smoothIx, box');
figure; imshow(smoothI2, []);

% Filtering
h = fspecial('gaussian', [1 15], 2)
figure; plot(h);
h = fspecial('gaussian', [1 50], 15)
figure; plot(h);
h = fspecial('gaussian', [1 25], 3)
smoothI2 = conv2(I2, h);
figure; imshow(smoothI2, []);
smoothI2 = conv2(I2, h');
figure; imshow(smoothI2, []);
close all;
close all; clc
h = [ -1 1 ];
derivX = conv2(double(I2), h);
figure; imshow(derivX, []);
figure; hist(derivX(:), 100);
derivY = conv2(double(I2), h');
figure; imshow(derivY, []);
%gradMag = sqrt( derivX.^2 + derivY.^2 );
whos
close all;
derivX = conv2(double(I2), h, 'same');
derivY = conv2(double(I2), h','same');
gradMag = sqrt( derivX.^2 + derivY.^2 );
figure; imshow(gradMag,[]);
figure; imshow(I2);

x = -200:200;
sigma = 21.5
G =  exp( - x.^2 / sigma^2 );
figure; plot(G);

% saving workspace
close all;
clc
whos
save currentEnv.mat
clear
whos
clc
whos
load currentEnv.mat
whos
clear
clc
whos
save currentEnv1.mat F h I1 I2
clear
whos
load currentEnv1.mat
whos
figure
clear; close all; clc; pack
for i = 10:-1:-10,
    disp(sprintf('%d', i));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;close all;clc;

% Image reading from disk.
I1 = imread('groceries.jpg');

figure;
imshow(I1);
keyboard;
[rows cols chan]=size(I1);

for i=1:rows
    disp(num2str(i));
end
    
x=testFunc(3,2)
